--[[

    Play a soundfile or/and vibrate for one switch, control, function, etc Event

    If the input is proportional, be abble to deffine a "Value ±Range" for the Event

--]]



------Variables
local Ver_BE
local lang_BE

local BE_Sw, BE_SwValue, BE_SwLastValue, BE_Snd, BE_Val, BE_NegTol, BE_PosTol
local BE_SwProp, BE_Event = false, false
local BE_Vib = {side=0, type=0}


local getSwitchInfo = system.getSwitchInfo
local getInputsVal = system.getInputsVal
local playFile = system.playFile
local vibration = system.vibration

--------------------------------------------------------------------------------
-- Read TRANSLATION
local function getTranslation()
  local trans = system.getLocale()
  local file  = io.readall("Apps/BttrEvent/lang/BttrEvent.jsn")
  local dec_file = json.decode(file)

  if (dec_file) then
    lang_BE = dec_file[trans] or dec_file[dec_file.default]
  end

collectgarbage()
end


-- Read TRANSLATION
--------------------------------------------------------------------------------



--------------------------------------------------------------------------------
-- App Functions
local function event()

    playFile(BE_Snd, AUDIO_IMMEDIATE)

    if (BE_Vib.type~=0 and BE_Vib.side==0) then       --LeftSide vibrations
      vibration(false, BE_Vib.type)
    elseif (BE_Vib.type~=0 and BE_Vib.side==1) then   --RightSide vibrations
      vibration(true, BE_Vib.type)
    end


collectgarbage()
end
-- App Functions
--------------------------------------------------------------------------------



--------------------------------------------------------------------------------
--Draw App FORM-----------------------------------------------------------------
local function Form_BE()
  local addRow = form.addRow
  local addSpacer = form.addSpacer
  local reinit = form.reinit
  local addLabel = form.addLabel
  local addInputbox = form.addInputbox
  local addTextbox = form.addTextbox
  local addIntbox = form.addIntbox
  local addAudioFilebox = form.addAudioFilebox
  local addSelectbox = form.addSelectbox
  local pSave = system.pSave

  --  Create Vibration Selection Table only when Form is started
  --  --> lower memory usage during runtime
  local BE_VibOpt = {lang_BE.vibType0, lang_BE.vibType1,lang_BE.vibType2,lang_BE.vibType3,lang_BE.vibType4}
    for i = 2, 5 do
      BE_VibOpt[i+4] = lang_BE.vibSide2..BE_VibOpt[i]
      BE_VibOpt[i]   = lang_BE.vibSide1..BE_VibOpt[i]
    end

  addSpacer(1,5)

  addRow(2)
  addLabel({label=lang_BE.selecIntput, font=FONT_NORMAL, width=250})
  addInputbox(BE_Sw, true, function(value)
    BE_Sw = value
    BE_SwProp = getSwitchInfo(BE_Sw).proportional --Check if selected Input Switch is proportional

    pSave("BE_Sw", BE_Sw)
    reinit(1)
  end)


  --If Input Switch is proportional --> get value & range for event
  if (BE_SwProp==true) then
    local BE_Range1, BE_Range2, BE_RangeTxt --to display the Event Range

    addSpacer(1,3)

    addRow(4)
    addLabel({label=lang_BE.valRange, font=FONT_NORMAL, width=145})

    addIntbox(BE_NegTol, -99, 0, 0, 0, 1, function(value)
      BE_NegTol = value
      pSave("BE_NegTol", value)
    end, {width=50})

    addIntbox(BE_Val, -100, 100, 0, 0, 1, function(value)
      BE_Val = value
      pSave("BE_Val", value)
    end, {label="%", width=70})

    addIntbox(BE_PosTol, 0, 99, 0, 0, 1, function(value)
      BE_PosTol = value
      pSave("BE_PosTol", value)
    end, {width=50})


    BE_Range1 = BE_Val+BE_NegTol
    BE_Range2 = BE_Val+BE_PosTol

    if ( BE_Range1<(-100) ) then
      BE_Range1 = -100
    elseif ( BE_Range2>100 ) then
      BE_Range2 = 100
    end

    BE_RangeTxt = "<< "..BE_Range1.."%  "..lang_BE.range.."  "..BE_Range2.."% >>"

    addSpacer(1,3)

    addLabel({label=BE_RangeTxt, font=FONT_MINI, alignRight=true})

    addLabel({label=lang_BE.rangeNote, font=FONT_MINI})
    addLabel({label=lang_BE.rangeNote1, font=FONT_MINI})
    addLabel({label=lang_BE.rangeNote2, font=FONT_MINI})


  end

  addSpacer(1,5)

  addRow(2)
  addLabel({label=lang_BE.snd, font=FONT_NORMAL})
  addAudioFilebox(BE_Snd, function(value)
    BE_Snd = value
    pSave("BE_Snd", value)
    reinit(1)
  end)

  addRow(2)
  addLabel({label=lang_BE.vib, font=FONT_NORMAL})
  addSelectbox(BE_VibOpt, 1+(BE_Vib.type + (4*BE_Vib.side)), true, function(value)
    BE_Vib.type = value
    if (value>=2 and value<=5) then --LeftSide vibrations (.side = 0)
      BE_Vib.side = 0
      BE_Vib.type = value - 1
    elseif (value>5) then           --RightSide vibrations (.side = 1)
      BE_Vib.side = 1
      BE_Vib.type = value - 5
    else                            --Vibrations OFF
      BE_Vib.type = 0
    end
    pSave("BE_Vib.side", BE_Vib.side)
    pSave("BE_Vib.type", BE_Vib.type)
    reinit(1)
  end)

  addSpacer(1,15)

  addRow(1)
  addLabel({label="by Rodrigo - v." .. Ver_BE, font=FONT_MINI, alignRight=true})

collectgarbage()
end
--Draw App FORM-----------------------------------------------------------------
--------------------------------------------------------------------------------


--------------------------------------------------------------------------------
--The LOOP----------------------------------------------------------------------
local function loop()

  if not BE_Sw then
    return  --Exit if no Report Controler has been choosen
  end

  BE_SwValue = getInputsVal(BE_Sw)

  --Input Switch is NOT proportional
  if (BE_SwProp==false) then
    if (BE_SwValue==1 and BE_Event==false) then

      event()
      BE_Event = true

    elseif (BE_SwValue==-1 and BE_Event==true) then

      BE_Event = false
    end

  end
  --Input Switch is NOT proportional




  --Input Switch is proportional
  if (BE_SwProp==true) then

    BE_SwValue = 100 * BE_SwValue

    if not BE_SwLastValue then
      BE_SwLastValue = BE_SwValue
    end


    if  not BE_Event
        and (BE_SwValue>=BE_Val+BE_NegTol and BE_SwLastValue<BE_Val+BE_NegTol)
        or  (BE_SwValue<=BE_Val+BE_PosTol and BE_SwLastValue>BE_Val+BE_PosTol) then

        event()
        BE_Event = true

    end

    if (BE_Event==true) and (BE_SwValue<BE_Val+BE_NegTol or BE_SwValue>BE_Val+BE_PosTol) then
      BE_Event = false
    end

    BE_SwLastValue = BE_SwValue

  end
  --Input Switch is proportional


collectgarbage()
end
--The LOOP----------------------------------------------------------------------
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
--App INITIALIZATION------------------------------------------------------------------
local function init()
  local pLoad = system.pLoad

  system.registerForm(1,MENU_APPS,lang_BE.appName_l, Form_BE)


  --Load saved App Data
  BE_Sw = pLoad("BE_Sw")
  BE_Val = pLoad("BE_Val", 0)
  BE_NegTol = pLoad("BE_NegTol", 0)
  BE_PosTol = pLoad("BE_PosTol", 0)
  BE_Snd = pLoad("BE_Snd","")

  BE_Vib.side = pLoad("BE_Vib.side", 0)
  BE_Vib.type = pLoad("BE_Vib.type", 0)

  --Check if selected Input Switch is proportional
  if (BE_Sw) then
    BE_SwProp = getSwitchInfo(BE_Sw).proportional
  end

end
--------------------------------------------------------------------------------
Ver_BE = "1.0"

getTranslation()
collectgarbage()
return {init=init, loop=loop, author="Rodrigo", version=Ver_BE, name=lang_BE.appName_s}
